from typing import Any, Iterator, List, Optional, Set, Union

from dev.thecesrom.helper.types import AnyStr
from java.io import BufferedReader, BufferedWriter, InputStream, OutputStream
from java.lang import AutoCloseable, CharSequence, Class, Enum, Object
from java.nio.channels import SeekableByteChannel
from java.nio.charset import Charset
from java.nio.file.attribute import (
    BasicFileAttributes,
    FileAttribute,
    FileAttributeView,
    FileStoreAttributeView,
    FileTime,
    PosixFilePermission,
    UserPrincipal,
)
from java.util.function import BiPredicate
from java.util.stream import Stream

class CopyOption: ...

class DirectoryStream(AutoCloseable):
    def close(self) -> None: ...

class OpenOption: ...

class Watchable:
    def register(self, *args: Any) -> WatchKey: ...

class FileStore(Object):
    def getAttribute(self, attribute: AnyStr) -> Object: ...
    def getBlockSize(self) -> long: ...
    def getFileStoreAttributeView(self, type: Class) -> FileStoreAttributeView: ...
    def getTotalSpace(self) -> long: ...
    def getUnallocatedSpace(self) -> long: ...
    def getUsableSpace(self) -> long: ...
    def isReadOnly(self) -> bool: ...
    def name(self) -> AnyStr: ...
    def supportsFileAttributeView(self, type: Class) -> bool: ...
    def type(self) -> AnyStr: ...

class FileVisitOption(Enum):
    FOLLOW_LINKS: FileVisitOption
    @staticmethod
    def values() -> List[FileVisitOption]: ...

class Files(Object):
    @staticmethod
    def copy(*args: Any) -> Union[long, Path]: ...
    @staticmethod
    def createDirectories(dir: Path, *attrs: FileAttribute) -> Path: ...
    @staticmethod
    def createDirectory(dir: Path, *attrs: FileAttribute) -> Path: ...
    @staticmethod
    def createFile(path: Path, *attrs: FileAttribute) -> Path: ...
    @staticmethod
    def createLink(link: Path, existing: Path) -> Path: ...
    @staticmethod
    def createSymbolicLink(link: Path, target: Path, *attrs: FileAttribute) -> Path: ...
    @staticmethod
    def createTempDirectory(*args: Any) -> Path: ...
    @staticmethod
    def createTempFile(*args: Any) -> Path: ...
    @staticmethod
    def delete(path: Path) -> None: ...
    @staticmethod
    def deleteIfExists(path: Path) -> bool: ...
    @staticmethod
    def exists(path: Path, *options: LinkOption) -> bool: ...
    @staticmethod
    def find(
        start: Path, maxDepth: int, matcher: BiPredicate, *options: FileVisitOption
    ) -> Stream: ...
    @staticmethod
    def getAttribute(path: Path, attribute: AnyStr, *options: LinkOption) -> Object: ...
    @staticmethod
    def getFileAttributeView(
        path: Path, type: Class, *options: LinkOption
    ) -> FileAttributeView: ...
    @staticmethod
    def getFileStore(path: Path) -> FileStore: ...
    @staticmethod
    def getLastModifiedTime(path: Path, *options: LinkOption) -> FileTime: ...
    @staticmethod
    def getOwner(path: Path, *options: LinkOption) -> UserPrincipal: ...
    @staticmethod
    def getPosixFilePermissions(
        path: Path, *options: LinkOption
    ) -> Set[PosixFilePermission]: ...
    @staticmethod
    def isDirectory(path: Path, *options: LinkOption) -> bool: ...
    @staticmethod
    def isExecutable(path: Path) -> bool: ...
    @staticmethod
    def isHidden(path: Path) -> bool: ...
    @staticmethod
    def isReadable(path: Path) -> bool: ...
    @staticmethod
    def isRegularFile(path: Path, *options: LinkOption) -> bool: ...
    @staticmethod
    def isSameFile(path: Path, path2: Path) -> bool: ...
    @staticmethod
    def isSymbolicLink(path: Path) -> bool: ...
    @staticmethod
    def isWritable(path: Path) -> bool: ...
    @staticmethod
    def lines(path: Path, cs: Optional[Charset] = ...) -> Stream: ...
    @staticmethod
    def list(path: Path) -> Stream: ...
    @staticmethod
    def move(source: Path, target: Path, *options: CopyOption) -> Path: ...
    @staticmethod
    def newBufferedReader(
        path: Path, cs: Optional[Charset] = ...
    ) -> BufferedReader: ...
    @staticmethod
    def newBufferedWriter(path: Path, *args: Any) -> BufferedWriter: ...
    @staticmethod
    def newByteChannel(path: Path, *args: Any) -> SeekableByteChannel: ...
    @staticmethod
    def newDirectoryStream(dir: Path, *args: Any) -> DirectoryStream: ...
    @staticmethod
    def newInputStream(path: Path, *options: OpenOption) -> InputStream: ...
    @staticmethod
    def newOutputStream(path: Path, *options: OpenOption) -> OutputStream: ...
    @staticmethod
    def notExists(path: Path, *options: LinkOption) -> bool: ...
    @staticmethod
    def probeContentType(path: Path) -> AnyStr: ...
    @staticmethod
    def readAllBytes(path: Path) -> bytearray: ...
    @staticmethod
    def readAllLines(path: Path, cs: Optional[Charset] = ...) -> List[AnyStr]: ...
    @staticmethod
    def readAttributes(path: Path, *args: Any) -> BasicFileAttributes: ...
    @staticmethod
    def readString(path: Path, cs: Optional[Charset] = ...) -> AnyStr: ...
    @staticmethod
    def readSymbolicLink(link: Path) -> Path: ...
    @staticmethod
    def setAttribute(
        path: Path, attribute: AnyStr, value: Object, *options: LinkOption
    ) -> Path: ...
    @staticmethod
    def setLastModifiedTime(path: Path, time: FileTime) -> Path: ...
    @staticmethod
    def setOwner(path: Path, owner: UserPrincipal) -> Path: ...
    @staticmethod
    def setPosixFilePermissions(
        path: Path, perms: Set[PosixFilePermission]
    ) -> Path: ...
    @staticmethod
    def size(path: Path) -> long: ...
    @staticmethod
    def walk(path: Path, *args: Any) -> Stream: ...
    @staticmethod
    def walkFileTree(path: Path, *args: Any) -> Path: ...
    @staticmethod
    def write(path: Path, *args: Any) -> Path: ...
    @staticmethod
    def writeString(path: Path, csq: CharSequence, *args: Any) -> Path: ...

class LinkOption(Enum, CopyOption, OpenOption):
    NOFOLLOW_LINKS: LinkOption
    @staticmethod
    def values() -> List[LinkOption]: ...

class Path(Watchable):
    def compareTo(self, other: Path) -> int: ...
    def endsWith(self, other: Union[Object, Path, AnyStr]) -> bool: ...
    def equals(self, other: Object) -> bool: ...
    def getFileName(self) -> Path: ...
    def getName(self, index: int) -> Path: ...
    def getNameCount(self) -> int: ...
    def getParent(self) -> Path: ...
    def getRoot(self) -> Path: ...
    def hashCode(self) -> int: ...
    def isAbsolute(self) -> bool: ...
    def iterator(self) -> Iterator[Path]: ...
    def normalize(self) -> Path: ...
    @staticmethod
    def of(*args: Any) -> Path: ...
    def register(self, *args: Any) -> WatchKey: ...
    def relativize(self, other: Path) -> Path: ...
    def resolve(self, other: Union[Path, AnyStr]) -> Path: ...
    def resolveSibling(self, other: Path) -> Path: ...
    def startsWith(self, other: Path) -> bool: ...
    def subpath(self, beginIndex: int, endIndex: int) -> Path: ...
    def toAbsolutePath(self) -> Path: ...
    def toFile(self) -> Any: ...
    def toRealPath(self, *args: LinkOption) -> Path: ...
    def toString(self) -> AnyStr: ...
    def toUri(self) -> Any: ...

class Paths(Object):
    @staticmethod
    def get(*args: Any) -> Path: ...

class StandardCopyOption(Enum, CopyOption, OpenOption):
    ATOMIC_MOVE: StandardCopyOption
    COPY_ATTRIBUTES: StandardCopyOption
    REPLACE_EXISTING: StandardCopyOption
    @staticmethod
    def values() -> List[StandardCopyOption]: ...

class WatchEvent:
    def context(self) -> Any: ...
    def count(self) -> int: ...
    def kind(self) -> WatchEvent.Kind: ...

    class Kind:
        def name(self) -> AnyStr: ...
        def type(self) -> Class: ...

    class Modifier:
        def name(self) -> AnyStr: ...

class WatchKey:
    def cancel(self) -> None: ...
    def isValid(self) -> bool: ...
    def pollEvents(self) -> List[WatchEvent]: ...
    def reset(self) -> bool: ...
    def watchable(self) -> Watchable: ...



from typing import List

from dev.thecesrom.helper.types import AnyStr

def loadFromFile(filePath: AnyStr, projectName: AnyStr, mode: int) -> None: ...
def removeGroups(projectName: AnyStr, paths: List[AnyStr]) -> None: ...



from dev.thecesrom.helper.types import AnyStr
from java.lang import Object
from java.util import EventObject

class PropertyChangeEvent(EventObject):
    def __init__(
        self, source: Object, propertyName: AnyStr, oldValue: Object, newValue: Object
    ) -> None: ...
    def getNewValue(self) -> Object: ...
    def getOldValue(self) -> Object: ...
    def getPropagationId(self) -> Object: ...
    def setPropagationId(self, propagationId: Object) -> None: ...

class PropertyChangeListener:
    def propertyChange(self, evt: PropertyChangeEvent) -> None: ...



from typing import Any, Iterator, List

from com.inductiveautomation.ignition.common import Dataset
from com.inductiveautomation.ignition.common.alarming import AlarmEvent, PyAlarmEvent
from dev.thecesrom.helper.types import AnyStr

class AlarmQueryResult:
    def getAssociatedDate(self, uuid: AnyStr) -> Dataset: ...
    def getDataSet(self) -> Dataset: ...
    def getEvent(self, uuid: AnyStr) -> AlarmEvent: ...

class AlarmQueryResultImpl(AlarmQueryResult):
    def __init__(self, *args: Any) -> None: ...
    def __iter__(self) -> Iterator[PyAlarmEvent]: ...
    @staticmethod
    def buildFrom(results: List[AlarmQueryResult]) -> AlarmQueryResult: ...
    def getAssociatedDate(self, uuid: AnyStr) -> Dataset: ...
    def getDataSet(self) -> Dataset: ...
    def getEvent(self, uuid: AnyStr) -> AlarmEvent: ...



from typing import Any, List

from com.inductiveautomation.ignition.common.tags.model import TagPath
from dev.thecesrom.helper.types import AnyStr
from java.lang import Object

class TagPathParser(Object):
    PARENT_RELATIVE: AnyStr
    PATH_SEPARATOR: AnyStr
    PROPERTY_SEPARATOR: AnyStr
    RELATIVE_DIR_UP: AnyStr
    ROOT_RELATIVE: AnyStr
    @staticmethod
    def chopPath(string: AnyStr) -> List[AnyStr]: ...
    @staticmethod
    def derelativize(tagPath: TagPath, relativeRoot: TagPath) -> TagPath: ...
    @staticmethod
    def isRelativePath(path: TagPath) -> bool: ...
    @staticmethod
    def parse(*args: Any) -> TagPath: ...
    @staticmethod
    def parseSafe(*args: Any) -> TagPath: ...
    @staticmethod
    def relativize(newPath: TagPath, relativeTo: TagPath) -> TagPath: ...



from typing import Any, List, Optional, Union

from dev.thecesrom.helper.types import AnyStr
from java.lang import CharSequence, Object, StringBuffer, StringBuilder
from java.util.function import Function, Predicate

class MatchResult:
    def end(self, group: Optional[int] = ...) -> int: ...
    def group(self, group: Optional[int] = ...) -> AnyStr: ...
    def groupCount(self) -> int: ...
    def start(self, group: Optional[int] = ...) -> int: ...

class Matcher(Object, MatchResult):
    def appendReplacement(
        self, sb: Union[StringBuffer, StringBuilder], replacement: AnyStr
    ) -> Matcher: ...
    def appendTail(
        self, sb: Union[StringBuffer, StringBuilder]
    ) -> Union[StringBuffer, StringBuilder]: ...
    def end(self, group: Optional[int] = ...) -> int: ...
    def find(self, start: Optional[int] = ...) -> bool: ...
    def group(self, group: Optional[Union[int, AnyStr]] = ...) -> AnyStr: ...
    def groupCount(self) -> int: ...
    def hasAnchoringBounds(self) -> bool: ...
    def hasTransparentBounds(self) -> bool: ...
    def hitEnd(self) -> bool: ...
    def lookingAt(self) -> bool: ...
    def matches(self) -> bool: ...
    def pattern(self) -> Pattern: ...
    @staticmethod
    def quoteReplacement(s: AnyStr) -> AnyStr: ...
    def region(self, start: int, end: int) -> Matcher: ...
    def regionEnd(self) -> int: ...
    def regionStart(self) -> int: ...
    def replaceAll(self, arg: Union[Function, AnyStr]) -> AnyStr: ...
    def replaceFirst(self, arg: Union[Function, AnyStr]) -> AnyStr: ...
    def requireEnd(self) -> bool: ...
    def reset(self, input: Optional[CharSequence] = ...) -> Matcher: ...
    def results(self) -> Any: ...
    def start(self, group: Optional[int] = ...) -> int: ...
    def toMatchResult(self) -> MatchResult: ...
    def useAnchoringBounds(self, b: bool) -> Matcher: ...
    def usePattern(self, newPattern: Pattern) -> Matcher: ...
    def useTransparentBounds(self, b: bool) -> Matcher: ...

class Pattern(Object):
    CANON_EQ: int
    CASE_INSENSITIVE: int
    COMMENTS: int
    DOTALL: int
    LITERAL: int
    MULTILINE: int
    UNICODE_CASE: int
    UNICODE_CHARACTER_CLASS: int
    UNIX_LINES: int
    def asMatchPredicate(self) -> Predicate: ...
    def asPredicate(self) -> Predicate: ...
    @staticmethod
    def compile(regex: AnyStr, flags: Optional[int] = ...) -> Pattern: ...
    def flags(self) -> int: ...
    def matcher(self, input: CharSequence) -> Matcher: ...
    @staticmethod
    def matches(regex: AnyStr, input: CharSequence) -> bool: ...
    def pattern(self) -> AnyStr: ...
    @staticmethod
    def quote(s: AnyStr) -> AnyStr: ...
    def split(
        self, input: CharSequence, limit: Optional[int] = ...
    ) -> List[AnyStr]: ...
    def splitAsStream(self, input: CharSequence) -> Any: ...



from typing import Any, Callable, Dict, List, Optional

from com.inductiveautomation.ignition.common import BasicDataset
from com.inductiveautomation.ignition.common.browsing import Results
from com.inductiveautomation.ignition.common.model.values import (
    BasicQualifiedValue,
    QualityCode,
)
from com.inductiveautomation.ignition.common.sqltags.history.annotations import (
    Annotation,
)
from dev.thecesrom.helper.types import AnyStr
from java.util import Date

DEFAULT_TIMEOUT_MILLIS: int
LEGACY_DEFAULT_TIMEOUT_MILLIS: int
TAG_PATH: Any

def browse(path: AnyStr, filter: Optional[Dict[AnyStr, Any]] = ...) -> Results: ...
def browseHistoricalTags(
    path: AnyStr,
    nameFilters: Optional[List[AnyStr]] = ...,
    maxSize: Optional[int] = ...,
    continuationPoint: Optional[Any] = ...,
) -> Results: ...
def configure(
    basePath: AnyStr, tags: List[Dict[AnyStr, Any]], collisionPolicy: AnyStr = ...
) -> List[QualityCode]: ...
def copy(
    tags: List[AnyStr], destination: AnyStr, collisionPolicy: AnyStr = ...
) -> List[QualityCode]: ...
def deleteAnnotations(
    paths: List[AnyStr], storageIds: List[AnyStr]
) -> List[BasicQualifiedValue]: ...
def deleteTags(tagPaths: List[AnyStr]) -> List[QualityCode]: ...
def exists(tagPath: AnyStr) -> bool: ...
def exportTags(
    filePath: Optional[AnyStr] = ...,
    tagPaths: Optional[List[AnyStr]] = ...,
    recursive: bool = ...,
    exportType: AnyStr = ...,
) -> Optional[AnyStr]: ...
def getConfiguration(
    basePath: AnyStr, recursive: bool = ...
) -> List[Dict[AnyStr, Any]]: ...
def importTags(
    filePath: AnyStr, basePath: AnyStr, collisionPolicy: AnyStr = ...
) -> List[QualityCode]: ...
def isOverlaysEnabled() -> bool: ...
def move(
    tags: List[AnyStr], destination: AnyStr, collisionPolicy: AnyStr = ...
) -> List[QualityCode]: ...
def query(
    provider: Optional[AnyStr] = ...,
    query: Optional[Dict[AnyStr, Any]] = ...,
    limit: Optional[int] = ...,
    continuation: Optional[AnyStr] = ...,
) -> Results: ...
def queryAnnotations(
    paths: List[AnyStr],
    startTime: Optional[Date] = ...,
    endTime: Optional[Date] = ...,
    types: Optional[List[AnyStr]] = ...,
) -> List[Annotation]: ...
def queryTagCalculations(
    paths: List[AnyStr],
    calculations: List[AnyStr],
    startDate: Optional[Date] = ...,
    endDate: Optional[Date] = ...,
    rangeHours: Optional[int] = ...,
    rangeMinutes: Optional[int] = ...,
    aliases: Optional[List[AnyStr]] = ...,
    includeBoundingValues: bool = ...,
    validatesSCExec: bool = ...,
    noInterpolation: bool = ...,
    ignoreBadQuality: bool = ...,
) -> BasicDataset: ...
def queryTagDensity(
    paths: List[AnyStr], startDate: Date, endDate: Date
) -> BasicDataset: ...
def queryTagHistory(
    paths: List[AnyStr],
    startDate: Optional[Date] = ...,
    endDate: Optional[Date] = ...,
    returnSize: int = ...,
    aggregationMode: AnyStr = ...,
    returnFormat: AnyStr = ...,
    columnNames: Optional[List[AnyStr]] = ...,
    intervalHours: Optional[int] = ...,
    intervalMinutes: Optional[int] = ...,
    rangeHours: Optional[int] = ...,
    rangeMinutes: Optional[int] = ...,
    aggregationModes: Optional[List[AnyStr]] = ...,
    includeBoundingValues: Optional[bool] = ...,
    validateSCExec: Optional[bool] = ...,
    noInterpolation: Optional[bool] = ...,
    ignoreBadQuality: Optional[bool] = ...,
    timeout: Optional[int] = ...,
    intervalSeconds: Optional[int] = ...,
    rangeSeconds: Optional[int] = ...,
) -> BasicDataset: ...
def readAsync(tagPaths: List[AnyStr], callback: Callable[..., Any]) -> None: ...
def readBlocking(
    tagPaths: List[AnyStr], timeout: int = ...
) -> List[BasicQualifiedValue]: ...
def rename(
    tag: AnyStr, newName: AnyStr, collisionPollicy: AnyStr = ...
) -> QualityCode: ...
def requestGroupExecution(provider: AnyStr, tagGroup: AnyStr) -> None: ...
def setOverlaysEnabled(enabled: bool) -> None: ...
def storeAnnotations(
    paths: List[AnyStr],
    startTimes: Optional[List[Date]] = ...,
    endTimes: Optional[List[Date]] = ...,
    types: Optional[List[Annotation]] = ...,
    data: Optional[List[AnyStr]] = ...,
    storageIds: Optional[List[int]] = ...,
    deleted: Optional[List[bool]] = ...,
) -> List[BasicQualifiedValue]: ...
def storeTagHistory(
    historyprovider: AnyStr,
    tagprovider: AnyStr,
    paths: List[AnyStr],
    values: List[Any],
    qualities: Optional[List[int]] = ...,
    timestamps: Optional[List[Date]] = ...,
) -> None: ...
def writeAsync(
    tagPaths: List[AnyStr],
    values: List[Any],
    callback: Optional[Callable[..., Any]] = ...,
) -> None: ...
def writeBlocking(
    tagPaths: List[AnyStr], values: List[Any], timeout: int = ...
) -> List[QualityCode]: ...



from typing import Iterable, List

from dev.thecesrom.helper.types import AnyStr
from java.lang import Enum, Object
from java.util import Date

class LicenseDetails:
    def checkFlag(self, key: AnyStr) -> bool: ...
    def getExpirationDate(self) -> Date: ...
    def getLicenseDetail(self, key: AnyStr) -> AnyStr: ...
    def getLicenseDetails(self, key: AnyStr) -> List[LicenseRestriction]: ...
    def getModuleId(self) -> AnyStr: ...
    def getVersion(self) -> int: ...
    def isPlatformDetails(self) -> bool: ...

class LicenseState:
    def getLicenseMode(self) -> LicenseMode: ...
    def getModuleLicense(self) -> ModuleLicense: ...
    def getPlatformLicense(self) -> LicenseDetails: ...
    def getTrialExpirationDate(self) -> Date: ...
    def isTrialExpired(self) -> bool: ...

class LicenseMode(Enum):
    @staticmethod
    def values() -> Iterable[LicenseMode]: ...

class LicenseRestriction(Object):
    def __init__(self, restrictionName: AnyStr, restrictionValue: AnyStr) -> None: ...
    def getrestrictionName(self) -> AnyStr: ...
    def getrestrictionValue(self) -> AnyStr: ...

class ModuleLicense(LicenseDetails):
    def getLicenseDetail(self, key: AnyStr) -> AnyStr: ...
    def getLicenseDetails(self, key: AnyStr) -> List[LicenseRestriction]: ...
    def getModuleId(self) -> AnyStr: ...
    def getVersion(self) -> int: ...



from typing import Any, Iterator, List, Optional

from dev.thecesrom.helper.types import AnyStr
from java.io import Writer
from java.lang import Number, Object

class JSONArray(Object):
    def __init__(self, arg: Any = ...) -> None: ...
    def get(self, index: int) -> Object: ...
    def getBoolean(self, index: int) -> bool: ...
    def getDouble(self, index: int) -> float: ...
    def getInt(self, index: int) -> int: ...
    def getJSONArray(self, index: int) -> JSONArray: ...
    def getJSONObject(self, index: int) -> JSONObject: ...
    def getLong(self, index: int) -> long: ...
    def getString(self, index: int) -> AnyStr: ...
    def isNull(self, index: int) -> bool: ...
    def join(self, separator: AnyStr) -> AnyStr: ...
    def length(self) -> int: ...
    def opt(self, index: int) -> Object: ...
    def optBoolean(self, index: int, defaultValue: Optional[bool] = ...) -> bool: ...
    def optDouble(self, index: int, defaultValue: Optional[float] = ...) -> float: ...
    def optInt(self, index: int, defaultValue: Optional[int] = ...) -> int: ...
    def optJSONArray(self, index: int) -> JSONArray: ...
    def optJSONObject(self, index: int) -> JSONObject: ...
    def optLong(self, index: int, defaultValue: Optional[long] = ...) -> long: ...
    def optString(self, index: int, defaultValue: Optional[AnyStr] = ...) -> AnyStr: ...
    def put(self, index: int, value: Any) -> JSONArray: ...
    def remove(self, index: int) -> Object: ...
    def toJSONObject(self, names: JSONArray) -> JSONObject: ...
    def toString(self, indentFactor: Optional[int] = ...) -> AnyStr: ...
    def write(self, writer: Writer) -> Writer: ...

class JSONObject(Object):
    NULL: Object
    def __init__(self, *args: Any) -> None: ...
    def accumulate(self, key: AnyStr, value: Object) -> JSONObject: ...
    def append(self, key: AnyStr, value: Object) -> JSONObject: ...
    @staticmethod
    def doubleToString(d: float) -> AnyStr: ...
    def get(self, key: AnyStr) -> Object: ...
    def getBoolean(self, key: AnyStr) -> bool: ...
    def getDouble(self, key: AnyStr) -> float: ...
    def getInt(self, key: AnyStr) -> int: ...
    def getJSONArray(self, key: AnyStr) -> JSONArray: ...
    def getJSONObject(self, key: AnyStr) -> JSONObject: ...
    def getLong(self, key: AnyStr) -> long: ...
    @staticmethod
    def getNames(arg: JSONObject) -> List[AnyStr]: ...
    def getString(self, key: AnyStr) -> AnyStr: ...
    def has(self, key: AnyStr) -> bool: ...
    def increment(self, key: AnyStr) -> JSONObject: ...
    def isNull(self, key: AnyStr) -> bool: ...
    def keys(self) -> Iterator[AnyStr]: ...
    def length(self) -> int: ...
    def names(self) -> JSONArray: ...
    @staticmethod
    def numberToString(n: Number) -> AnyStr: ...
    def opt(self, key: AnyStr) -> Object: ...
    def optBoolean(self, key: AnyStr, defaultValue: Optional[bool] = ...) -> bool: ...
    def optDouble(self, key: AnyStr, defaultValue: Optional[float] = ...) -> float: ...
    def optInt(self, key: AnyStr, defaultValue: Optional[int] = ...) -> int: ...
    def optJSONArray(self, key: AnyStr) -> JSONArray: ...
    def optJSONObject(self, key: AnyStr) -> JSONObject: ...
    def optLong(self, key: AnyStr, defaultValue: Optional[long] = ...) -> long: ...
    def optString(
        self, key: AnyStr, defaultValue: Optional[AnyStr] = ...
    ) -> AnyStr: ...
    def put(self, key: AnyStr, value: Any) -> JSONObject: ...
    def putOnce(self, key: AnyStr, value: Object) -> JSONObject: ...
    def putOpt(self, key: AnyStr, value: Object) -> JSONObject: ...
    @staticmethod
    def quote(string: AnyStr) -> AnyStr: ...
    def remove(self, key: AnyStr) -> Object: ...
    def sortedKeys(self) -> Iterator[AnyStr]: ...
    @staticmethod
    def stringToValue(s: AnyStr) -> Object: ...
    def toJSONArray(self, names: JSONArray) -> JSONArray: ...
    def toString(self, indentFactor: Optional[int] = ...) -> AnyStr: ...
    @staticmethod
    def valueToString(value: Object) -> AnyStr: ...
    @staticmethod
    def wrap(obj: Object) -> Object: ...
    def write(self, writer: Writer) -> Writer: ...



from typing import Optional, Tuple

from dev.thecesrom.helper.types import AnyStr
from java.util import EventObject

def getRoles() -> Tuple[AnyStr, ...]: ...
def getUserRoles(
    username: AnyStr, password: AnyStr, authProfile: AnyStr = ..., timeout: int = ...
) -> Optional[Tuple[AnyStr, ...]]: ...
def getUsername() -> AnyStr: ...
def isScreenLocked() -> bool: ...
def lockScreen(obscure: bool = ...) -> None: ...
def logout() -> None: ...
def switchUser(
    username: AnyStr, password: AnyStr, event: EventObject, hideError: bool = ...
) -> bool: ...
def unlockScreen() -> None: ...
def validateUser(
    username: AnyStr, password: AnyStr, authProfile: AnyStr = ..., timeout: int = ...
) -> bool: ...



from typing import List

from dev.thecesrom.helper.types import AnyStr

def getProjectName() -> AnyStr: ...
def getProjectNames() -> List[AnyStr]: ...
def requestScan(timeout: int = ...) -> None: ...



from typing import Any, Dict, Optional, Union

from com.inductiveautomation.factorypmi.application import FPMIWindow
from com.inductiveautomation.factorypmi.application.script.builtin import NavUtilities
from dev.thecesrom.helper.types import AnyStr
from java.util import EventObject

def centerWindow(arg: Union[AnyStr, FPMIWindow]) -> None: ...
def closeParentWindow(event: EventObject) -> None: ...
def closeWindow(arg: Union[AnyStr, FPMIWindow]) -> None: ...
def desktop(handle: Optional[AnyStr] = ...) -> NavUtilities: ...
def getCurrentWindow() -> AnyStr: ...
def goBack() -> FPMIWindow: ...
def goForward() -> FPMIWindow: ...
def goHome() -> FPMIWindow: ...
def openWindow(
    path: AnyStr, params: Optional[Dict[AnyStr, Any]] = ...
) -> FPMIWindow: ...
def openWindowInstance(
    path: AnyStr, params: Optional[Dict[AnyStr, Any]] = ...
) -> FPMIWindow: ...
def swapTo(path: AnyStr, params: Optional[Dict[AnyStr, Any]] = ...) -> FPMIWindow: ...
def swapWindow(
    arg: Union[AnyStr, EventObject],
    swapToPath: AnyStr,
    params: Optional[Dict[AnyStr, Any]] = ...,
) -> FPMIWindow: ...



from typing import Optional, Union

from dev.thecesrom.helper.types import AnyStr
from java.util import Date, Locale

def addDays(date: Date, value: int) -> Date: ...
def addHours(date: Date, value: int) -> Date: ...
def addMillis(date: Date, value: int) -> Date: ...
def addMinutes(date: Date, value: int) -> Date: ...
def addMonths(date: Date, value: int) -> Date: ...
def addSeconds(date: Date, value: int) -> Date: ...
def addWeeks(date: Date, value: int) -> Date: ...
def addYears(date: Date, value: int) -> Date: ...
def daysBetween(date_1: Date, date_2: Date) -> int: ...
def format(date: Date, format: AnyStr = ...) -> unicode: ...
def fromMillis(millis: int) -> Date: ...
def getAMorPM(date: Date) -> int: ...
def getDate(year: int, month: int, day: int) -> Date: ...
def getDayOfMonth(date: Date) -> int: ...
def getDayOfWeek(date: Date) -> int: ...
def getDayOfYear(date: Date) -> int: ...
def getHour12(date: Date) -> int: ...
def getHour24(date: Date) -> int: ...
def getMillis(date: Date) -> int: ...
def getMinute(date: Date) -> int: ...
def getMonth(date: Date) -> int: ...
def getQuarter(date: Date) -> int: ...
def getSecond(date: Date) -> int: ...
def getTimezone() -> AnyStr: ...
def getTimezoneOffset(date: Optional[Date] = ...) -> float: ...
def getTimezoneRawOffset() -> float: ...
def getYear(date: Date) -> int: ...
def hoursBetween(date_1: Date, date_2: Date) -> int: ...
def isAfter(date_1: Date, date_2: Date) -> bool: ...
def isBefore(date_1: Date, date_2: Date) -> bool: ...
def isBetween(target_date: Date, start_date: Date, end_date: Date) -> bool: ...
def isDaylightTime(date: Optional[Date] = ...) -> bool: ...
def midnight(date: Date) -> Date: ...
def millisBetween(date_1: Date, date_2: Date) -> long: ...
def minutesBetween(date_1: Date, date_2: Date) -> int: ...
def monthsBetween(date_1: Date, date_2: Date) -> int: ...
def now() -> Date: ...
def parse(
    dateString: AnyStr,
    formatString: AnyStr = ...,
    locale: Union[AnyStr, Locale, None] = ...,
) -> Date: ...
def secondsBetween(date_1: Date, date_2: Date) -> int: ...
def setTime(date: Date, hour: int, minute: int, second: int) -> Date: ...
def toMillis(date: Date) -> long: ...
def weeksBetween(date_1: Date, date_2: Date) -> int: ...
def yearsBetween(date_1: Date, date_2: Date) -> int: ...
